%define releasebase 7
%define nbase fftw
%define prefix /opt/mpi/%{nbase}/%{version}/%{compilertag}/%{mpitag}
%define modulepath /opt/Modules/modulefiles/

#Use as deafults if no command line inputs
%define as_limulus 1
%define use_gnu4  1
#  NOTE: MPI's are mutually exclusive, only set one,
#        if not using command line options
#        Right now they only work with gnu 
#        gcc3.x and gcc4.x, it will work with pgi and mpichgm-pgi
#        other compilers and have not been thoroughly tested or configured
%define use_mpich2 0
%define use_ompi  1 
# allow --with[out] <feature> at rpm command line build
%{?_with_basement: %{expand: %%define as_basement 1}}
%{?_with_limulus: %{expand: %%define as_limulus 1}}
#possible Compilers
%{?_with_gnu3: %{expand: %%define use_gnu3 1}}
%{?_with_gnu4: %{expand: %%define use_gnu4 1}}
%{?_with_pgi: %{expand: %%define use_pgi 1}}
%{?_with_intel: %{expand: %%define use_intel 1}}
%{?_with_pathsc: %{expand: %%define use_pathsc 1}}
# possible MPI's
%{?_with_mpich2: %{expand: %%define use_mpich2 1}}
%{?_with_mpich2omx: %{expand: %%define use_mpich2omx 1}}
%{?_with_mpich2ib: %{expand: %%define use_mpich2ib 1}}
%{?_with_ompi: %{expand: %%define use_ompi 1}}
#pick the target
%if "1" == "%{as_basement}"
%define targetbase Basement
%else
%define targetbase Limulus
%endif
%global _compiler_load  module load %{compilertag}
%global _compiler_unload  module unload %{compilertag}
%global _mpi_load module load %{mpitag}/%{mpiversion}/%{compilertag};
%global _mpi_unload  module unload %{mpitag}/%{mpiversion}/%{compilertag};

# pick the compiler
%if "1" == "%{use_gnu3}"
%define compilertag gnu3
%define setenvs export CC=gcc; export CFLAGS='-O3 -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'; export F77=g77; export FFLAGS='-O3';
%else
%if "1" == "%{use_pgi}"
%define compilertag pgi
%define setenvs export CC=pgcc; export F77=pgf77; export CFLAGS='-fastsse -O3 '; export FFLAGS='-fastsse -O3 ';
%else
%if "1" == "%{use_intel}"
%define compilertag intel
%define setenvs export F77=ifort;   export FFLAGS='-O3 ';export CC=icc;  export CFLAGS='-O3';
%else
%if "1" == "%{use_pathsc}"
%define compilertag pathsc
echo "Path Comp not implemented"
exit
%else
%define compilertag gnu4
%define setenvs export CC=gcc; export CFLAGS='-O3 -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'; export F77=gfortran; export FFLAGS=' -O3'; 
%endif
%endif
%endif
%endif
# set the arch variables
%ifarch i386
%define fftw2hacks   --enable-i386-hacks
%endif
%ifarch X86_64
%define fftw2hacks --disable-i386-hacks
%endif

# pick the MPI
%if "1" == "%{use_ompi}"
%define mpitag openmpi 
%define mpiversion 1.6.3
%endif
%if "1" == "%{use_mpich2omx}"
%define mpitag mpich2-omx
%define mpiversion 1.4.1p1
%endif
%if "1" == "%{use_mpich2ib}"
%define mpitag mpich2-ib
%define mpiversion 1.4.1p1
%endif
%if "1" == "%{use_mpich2}"
%define mpitag mpich2
%define mpiversion 1.4.1p1
%endif


Name: %{nbase}-%{mpitag}-%{compilertag}
Summary: Fast Fourier Transform library
Version: 2.1.5
Group: %{targetbase}/Libraries
Release: %{releasebase}.%{targetbase}
License: GPL
URL: http://www.fftw.org/
Packager: deadline@basement-supercomputing.com
Vendor: Basement-Supercomputing.com
Source: %{nbase}-%{version}.tar.gz
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
Requires: modules  %{nbase}-%{compilertag}=%{version} %{mpitag}-%{compilertag}
BuildRequires: modules  %{mpitag}-%{compilertag}



%description
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package only contains the MPI libraries for fftw-2

%prep
%setup -q -c  %{nbase}-%{version}
mv %{nbase}-%{version} single
cp -a single double


%build
#we need modules to build this
rpm -qf /opt/Modules/bin/modulecmd >& /dev/null
if [ $? -ne 0 ]; then
  echo "This RPM requries the BSC Modules package to build"
fi
. /etc/profile.d/modules.sh
%{_compiler_load}
%{_mpi_load}
%setenvs

### build double
pushd double
./configure --prefix=%{prefix}   %{fftw2hacks} \
	--enable-mpi \
	 --with-pentium-timer  
%{__make} %{?_smp_mflags}
popd
pushd single
### build single
./configure --prefix=%{prefix}  %{fftw2hacks} \
	--enable-mpi \
	--with-pentium-timer  \
	--enable-type-prefix \
	--enable-float 
%{__make} %{?_smp_mflags}
popd
%{_compiler_unload}
%{_mpi_unload}

%install
#all we care about is the library, the fftw-2 package should already be installed
pushd double
        /bin/mkdir -p  %{buildroot}/%{prefix}/lib
	/bin/cp mpi/.libs/*_mpi.a  %{buildroot}/%{prefix}/lib
popd
pushd single
        /bin/mkdir -p  %{buildroot}/%{prefix}/lib
	/bin/cp mpi/.libs/*_mpi.a  %{buildroot}/%{prefix}/lib
popd

# create /opt//etc/ld.so.conf.d/
echo "Creating /opt//etc/ld.so.conf.d/%{name} file"
mkdir -p %{buildroot}/opt/etc/ld.so.conf.d
cat <<EOF> %{buildroot}/opt/etc/ld.so.conf.d/%{name}-%{version}.conf
%{prefix}/lib
EOF

#create the module file
mkdir -p %{buildroot}/%{modulepath}/%{nbase}-mpi/%{version}/%{compilertag}
cat <<EOF >>%{buildroot}/%{modulepath}/%{nbase}-mpi/%{version}/%{compilertag}/%{mpitag}
#%Module1.0#######################################################################
## %{nbase}-mpi modulefile
##

prereq %{nbase}/%{version}/%{compilertag}

proc ModulesHelp { } {
        puts stderr "\tThis module loads the appropriate environment for "
        puts stderr "\tusing fftw-%{mpitag}-%{compilertag} version %{version}.\n"
}

module-whatis "fftw-%{mpitag}-%{compilertag} %{version}"
module-log error none

set fftw2root %{prefix}

if [ module-info mode load ] {
  prepend-path  LD_LIBRARY_PATH \$fftw2root/lib
	setenv FFTW2_MPI_PATH \$fftw2root/lib/
	setenv FFTW2_INCLUDE \$fftw2root/include
  append-path INFOPATH \$fftw2root/info
}
if [ module-info mode remove ] {
  remove-path LD_LIBRARY_PATH \$fftw2root/lib
	unsetenv FFTW2_MPI_PATH 
	unsetenv FFTW2_INCLUDE
  remove-path INFOPATH \$fftw2root/info
}
EOF
%post
/sbin/ldconfig 2>/dev/null

%postun
/sbin/ldconfig 2>/dev/null


%clean
%{__rm} -rf %{buildroot}

%files
%defattr(-, root, root, 0755)
%{prefix}/lib/*.a
%{modulepath}/%{nbase}-mpi/%{version}/%{compilertag}/%{mpitag}
/opt/%{_sysconfdir}/ld.so.conf.d/%{name}-%{version}.conf

%changelog
* Fri Dec 12 2012  Douglas Eadline <deadline@basement-supercomuting.com>
- added /opt/etc/ld.so.conf.d entry
* Tue Sep 25 2012 - Douglas Eadline  <deadline@basement-supercomputing.com>
- module clean up

